///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//////////////////////                         ////////////////////////
//////////////////////           Balance       ////////////////////////
//////////////////////                         ////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
//
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
//
//	FreeRTOS includes
#include "FreeRTOS.h"
#include "task.h"

#include "parameters.h"

#include "balance.h"
#include "tilt.h"
#include "drivers.h"
#include "routines.h"
#include "globals.h"
#include "tilt.h"


// Included temporarily, until "timer_enable_int()" is replaced
#include <compat/deprecated.h>
//
//---------------------------------------------------------------------------------------------------------------------
#define DEBUG

//#define KALMAN		// theoretically optimal filtering, baby!

#define CPUCLK   CPUCLK_PARAM

#define MAX_TORQUE 4000

//---------------------------------------------------------------------------------------------------------------------

// Inputs
float angle;			// radians, [-pi/2,pi/2]
//float angle_rate;		// radians/sec, [-pi/2,pi/2] limited by rate gyro
float steer_knob; 		// unitless, range [-1,1]

volatile unsigned int accelRaw;
volatile unsigned int gyroRaw;
volatile unsigned int potRaw;

volatile float accel;
volatile float gyro;
volatile float pot;

// Kalman Filtering
// defined in tilt.c
extern float	angle;
extern float	q_bias;
extern float	rate;

extern float scale_accel(int rawAccel);
extern float scale_gyro(int rawGyro);


// Balance
float balance_torque;
float overspeed;
float overspeed_integral;
float gyro_integrated;

//	Proportional Konstant
float Kp = Kp_PARAM;						// balance loop P gain
//	Differentiate konstant
float Kd = Kd_PARAM;						// balance loop D gain 


//float Ksteer;								// steering control gain
//float Ksteer2;							// steering control tilt sensitivity
//float Kspeed; 							// speed tracking gain
float neutral = neutral_PARAM;						// "angle of natural balance"


float KpTurn;								// turn rate loop P gain
float KiTurn;								// turn rate loop I gain
float KdTurn;								// turn rate loop D gain

// Outputs
int left_motor_torque;
int right_motor_torque;

unsigned char balance_flag = FALSE;

int count = 0;

float current_angle = 0;
float current_rate = 0;

int	left_motor_bias = lmb_PARAM;
int right_motor_bias = rmb_PARAM;

//-----------------------------------------------------------------------------------------------------------
//
//
void balance(void)
{
	//balance_flag = FALSE;

	gyroRaw = ADC4_Val;
	accelRaw = ADC5_Val;

	gyro = scale_gyro(gyroRaw);
	accel = scale_accel(accelRaw);

	// Have measurement, find error and update K
	// no scaling needed.. but I'm doing it anyway

	kalman_update(accelRaw);


	// Produce new current angle and rate estimate
	// real units assumed (they're pretty close)

	state_update((float)(int)( gyroRaw - GYRO_OFFSET));

	// If tilt is positive, move wheels forward to balance;
	// + cmds are cw (right side, fwd)
	//				 (left side, bkwd)
	// So left side gets - cmds

	// This is just a casualty of the sensor mounting
	current_angle = -angle;
	current_rate = -rate;

	// Balance.  The most important line in the entire program.
	balance_torque = Kp * (current_angle - neutral) + Kd * current_rate;

	steer_knob = 0;

	// change from current angle to something proportional to speed
	// should this be the abs val of the cur speed or just curr speed?
	//float steer_cmd = (1.0 / (1.0 + Ksteer2 * fabs(current_angle))) * (Ksteer * steer_knob);
	float steer_cmd = 0.0;

	// Get current rate of turn
	//float current_turn = left_speed - right_speed; //<-- is this correct
	//float turn_accel = current_turn - prev_turn;
	//prev_turn = current_turn;

	// Closed-loop turn rate PID
	//float steer_cmd = KpTurn * (current_turn - steer_desired)
	//					+ KdTurn * turn_accel;
	//					//+ KiTurn * turn_integrated;

	// Possibly optional
	//turn_integrated += current_turn - steer_cmd;

	//	Differential steering
	left_motor_torque = balance_torque + steer_cmd; //+ cur_speed + steer_cmd;
	right_motor_torque = balance_torque - steer_cmd; //+ cur_speed - steer_cmd;


	// Limit extents of torque demand
	//left_motor_torque = flim(left_motor_torque, -MAX_TORQUE, MAX_TORQUE);
	if (left_motor_torque < -MAX_TORQUE) left_motor_torque = -MAX_TORQUE;
	if (left_motor_torque > MAX_TORQUE)  left_motor_torque =  MAX_TORQUE;

	//right_motor_torque = flim(right_motor_torque, -MAX_TORQUE, MAX_TORQUE);
	if (right_motor_torque < -MAX_TORQUE) right_motor_torque = -MAX_TORQUE;
	if (right_motor_torque > MAX_TORQUE)  right_motor_torque =  MAX_TORQUE;

	// Set PWM values for both motors
	if (abs(current_angle) < 45) {				//  check to see if bot has fallen over
		set_left_motor_pwm(left_motor_torque + left_motor_bias);
		set_right_motor_pwm(right_motor_torque + right_motor_bias);
	} else {
		set_left_motor_pwm(0);
		set_right_motor_pwm(0);
	}


}


