///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//////////////////////                         ////////////////////////
//////////////////////     Drivers Routines    ////////////////////////
//////////////////////                         ////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

#include "drivers.h"
#include "globals.h"
#include "bbot.h"

#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////


extern unsigned char	AD_Comp;		//  flag for ADC complete
extern unsigned char 	Sense_Stat;
extern unsigned int 	Rem_Value;	//	for ADC

extern unsigned int		Lsense_Val;
extern unsigned int		Rsense_Val;
extern unsigned int 	LDsense_Val;
extern unsigned int 	RDsense_Val;

extern unsigned int 	ADC4_Val;		//
extern unsigned int 	ADC5_Val;		//

extern unsigned int		Object_found;

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////


/*----------------------------------------------------------------------*/
/*  Function  : Led()                                                   */
/*  Action    : Turn on or off the programmed LED                       */
/*----------------------------------------------------------------------*/
void Led(char swtch) {

	if (swtch) {
		PORTB &= ~PROG_LED; 		// Turn LED on
		//PORTB &=~0x10;
		LED_ON = TRUE;
	} else {
		PORTB |= PROG_LED;  		// Turn LED off
		//PORTB |=0x10;
		LED_ON = FALSE;
	}
}




/*----------------------------------------------------------------------*/
/*  Function  : ButtonPressed()                                   		*/
/*  Action    : True if Start Switch (pb2) pressed                    	*/
/*----------------------------------------------------------------------*/
unsigned char ButtonPressed() {
	static unsigned char Latch = 0;

#ifdef IGNORE_START_BUTTON
	return TRUE;				//	if ignore is defined, return as if pressed
#endif

	if (Latch)
		return TRUE;
	if (PINB & START_SW) {		//  Button must be pressed for at least 5ms
		return FALSE;
	} else {
		Latch = TRUE;
		return TRUE;
	}
}



/*----------------------------------------------------------------------*/
/*  Function  : Init_Hdw()                                              */
/*  Action    : sets up the hardware                                    */
/*----------------------------------------------------------------------*/
void Init_Hdw()
{
	WDTCR =	0x18;			//	disable watchdog for now
	WDTCR = 0x00;
	ADMUX = 0xC0;			//  Aref 2.56v, 10bits
	//ADMUX = 0x00;			//	Aref 5.00v  (10 bits)

	//ADCSRA = 0x8E;		//	ad on,int on % 16 clk
	ADCSRA = 0x8C;			//

	PORTA  = 0x00;			//	no pullups on AD inputs
	DDRA   = 0x00;			//	A is AD0-7

	PORTB = 0xff;			//	pullups on port b
	DDRB  = 0xfc;			//	set d (pin 0 and 1)

	DDRC   = 0xFF;			//	C is AD 8-15
	PORTC =  0x00;			//	

	DDRD   = 0xF9;			//	direction of D
	PORTD  = 0x00;			//	ext rs232 port

	MCUCR  = 0x00;			//	no external ints


	GICR |= 0x01;
	GICR  = 0x00;			//	no ext pins

	TIMSK = 0;
	SREG = 0x80;			//	master ints on


}



/*----------------------------------------------------------------------*/
/*  Function  : Init_Pwm()                                              */
/*  Action    : Initializes the PWM hardware                            */
/*----------------------------------------------------------------------*/
void Init_Pwm()
{
	//	set 8 bit fast pwm, 2 channels
    TCCR1A = 	_BV(COM1A1) | _BV(COM1B1) | _BV(WGM10);
	//	select a PWM frequency
    TCCR1B = 	_BV(CS10)                 | _BV(WGM12);     // CLK/1
    //TCCR1B = 	_BV(CS11)                 | _BV(WGM12);     // CLK/8
    //TCCR1B = 	_BV(CS10) | _BV(CS11)     | _BV(WGM12);     // CLK/64
    //TCCR1B = 	_BV(CS12)                 | _BV(WGM12);     // CLK/256
    //TCCR1B = 	_BV(CS10) | _BV(CS12)     | _BV(WGM12);     // CLK/1024

    DDRD |= 	LEFT_MOTOR_PWM | RIGHT_MOTOR_PWM;
    DDRC |= 	LEFT_MOTOR_DIR | RIGHT_MOTOR_DIR;
    LEFT_PWM = 	RIGHT_PWM = 0;
}



/*-----------------------------------------------------------------------*/
/*  Function : ADC interrupt handler                                     */
/*  Action   : Sets conversion complete flag and resets conversion       */
/*-----------------------------------------------------------------------*/
SIGNAL(SIG_ADC)
{
	unsigned int temp;

	Rem_Value = (unsigned int)ADCL; 	// get low byte
 	temp = (unsigned int)ADCH;			// get high byte first
	Rem_Value |= (temp<<8);				// merge

  	AD_Comp = TRUE;                 	// flag ready
}




//----------------------------------------------------------//
//	Function	: Dist_Sensors()							//
//	Action		: reads Distance sensor(s) and filter		//
//----------------------------------------------------------//
void	Dist_Sensors()
{
	unsigned char mux;

	mux = ADMUX;					// store current settings

	AD_Comp=FALSE;
	ADMUX =mux|AD_DISTL;
	ADCSRA |= 0x10;            		// clr int flag
	ADCSRA |= _BV(ADSC);
	while(!AD_Comp);
	LDsense_Val=Rem_Value;

	if (LDsense_Val > 200)
		Sense_Stat |= SENSE_DISTL; 	//store dist status
	else
		Sense_Stat &= ~SENSE_DISTL;

	//AD_Comp=FALSE;
	//ADMUX =mux|AD_DISTR;
	//ADCSRA |= 0x10;            	// clr int flag
	//ADCSRA |= _BV(ADSC);
	//while(!AD_Comp);
	//RDsense_Val=Rem_Value;

	//if (LDsense_Val > 200)
	//	Sense_Stat |= SENSE_DISTR; 	//store dist status
	//else
	//	Sense_Stat &= ~SENSE_DISTR;


	ADMUX=mux; // restore state
}





//----------------------------------------------------------//
//	Function	: Clear_Dist_Sensors()	            		//
//	Action		: Clears latched dist sensor(s) 		    //
//----------------------------------------------------------//
void	Clear_Dist_Sensors()
{
	//	Clear dist sensor bits in stat
	Sense_Stat &= 0xF3;
}



/*
//----------------------------------------------------------//
//	Function	: Set_Left_Pwm(BYTE dir,BYTE Pval)			//
//	Action		: Sets the left pwm for direction and pulse //
//----------------------------------------------------------//
void Set_Left_Pwm(unsigned char dir,unsigned char Pval)
{
  if(dir==FWD)  //1
   {
     PORTC &=~LEFT_MOTOR_DIR;
     TCCR1A &= (unsigned char)~_BV(COM1B0); // low pwm output
     LEFT_PWM = Pval;
    }
  else
   {
    PORTC |= LEFT_MOTOR_DIR;
    TCCR1A |= _BV(COM1B0); //high pwm output
    LEFT_PWM = Pval;
   }
}

//----------------------------------------------------------//
//	Function	: Set_Left_Pwm(BYTE dir,BYTE Pval)			//
//	Action		: Sets the left pwm for direction and pulse //
//  				Right motor uses OCR1A, PC7				//
//----------------------------------------------------------//
void Set_Right_Pwm(unsigned char dir,unsigned char Pval)
{
  if(dir==FWD)
   {
    PORTC &=~RIGHT_MOTOR_DIR;
    TCCR1A &= (unsigned char)~_BV(COM1A0);
    RIGHT_PWM = Pval;
 	}
  else
   {
	PORTC |= RIGHT_MOTOR_DIR;
    TCCR1A |= _BV(COM1A0);
    RIGHT_PWM = Pval;
   }
}
*/

///////////////////////////////////////////////////////////////////////

//----------------------------------------------------------//
//	Function	: Read_ADC4_5()								//
//	Action		: reads accelerometers						//
//----------------------------------------------------------//
void	Read_ADC4_5()
{
	unsigned char mux;
	unsigned int ADC4_tmp;
	unsigned int ADC5_tmp;

	mux = ADMUX & 0xF8;			// read current mux settings	


	AD_Comp=FALSE;				// reset flag
	ADMUX = mux|AD_C4;			// set mux to ADC4
	ADCSRA |= 0x10;            	// clr int flag
	//ADCSRA |= _BV(ADSC);		// start conversion
	ADCSRA |= 0x40;				// start conversion
	while(!AD_Comp);			// wait for interrupt routine to finish
	ADC4_tmp=Rem_Value;

	AD_Comp=FALSE;				// reset flag
	ADMUX = mux|AD_C5;			// set mux to ADC5
	ADCSRA |= 0x10;            	// clr int flag
	//ADCSRA |= _BV(ADSC);		// start conversion
	ADCSRA |= 0x40;				// start conversion
	while(!AD_Comp);			// wait for interrupt routine to finish
	ADC5_tmp=Rem_Value;

	//	Make safe copy in case polled from seperate thread in RTOS
	
	ADCSRA  &= 0xf7;			//	off
	ADC4_Val = ADC4_tmp;
	ADC5_Val = ADC5_tmp;
	ADCSRA  |= 0x08;			//	on
	
}

///////////////////////////////////////////////////////////////////////

int clip_drive(int drive)
{
    if (drive > 255) drive = 255;
	if (drive < -255)drive = -255;
    return drive;
}



void set_left_motor_pwm(int drive)
{
	drive = clip_drive(drive);
#if 0
   if (drive > 0)
    {
        PORTC |= LEFT_MOTOR_DIR;
        TCCR1A |= _BV(COM1B0);
        drive = -drive;
 
 	}
    else
    {
        PORTC &= (unsigned char)~LEFT_MOTOR_DIR;
        TCCR1A &= (unsigned char)~_BV(COM1B0);
     } 
#endif
    if (drive > 0)
    {
        PORTC &= (unsigned char)~LEFT_MOTOR_DIR;
        TCCR1A &= (unsigned char)~_BV(COM1B0);
	}
    else
    {
        PORTC |= LEFT_MOTOR_DIR;
        TCCR1A |= _BV(COM1B0);
        drive = -drive;
     }

    LEFT_PWM = drive;
}




void set_right_motor_pwm(int drive)
{
	drive = clip_drive(drive);
#if 0
    if (drive > 0)
    {
       PORTC |= RIGHT_MOTOR_DIR;
       TCCR1A |= _BV(COM1A0);
	   drive = -drive;
     }
    else
    {
       PORTC &= (unsigned char)~RIGHT_MOTOR_DIR;
       TCCR1A &= (unsigned char)~_BV(COM1A0);
 
    } 
 #endif
    if (drive > 0)
    {
        PORTC &= (unsigned char)~RIGHT_MOTOR_DIR;
        TCCR1A &= (unsigned char)~_BV(COM1A0);
    }
    else
    {
        PORTC |= RIGHT_MOTOR_DIR;
        TCCR1A |= _BV(COM1A0);
		drive = -drive;
    }


    RIGHT_PWM = drive;
}


