///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//////////////////////                         ////////////////////////
//////////////////////          TASKS          ////////////////////////
//////////////////////                         ////////////////////////
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
//
//
//
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "FreeRTOS.h"
#include "task.h"

#include "globals.h"
#include "robotTasks.h"
#include "Routines.h"
#include "bbot.h"
#include "drivers.h"
#include "balance.h"
#include "twi.h"
#include "lcd.h"

#include "parameters.h"

///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////

extern void 	Read_ADC4_5(void);
extern void 	balance(void);

extern float 	accel;
extern float 	gyro;
extern float	angle;
extern float	q_bias;
extern float	rate;

extern float 	current_angle;

extern int 		left_motor_torque;
extern int 		right_motor_torque;
extern int		left_motor_bias;
extern int		right_motor_bias;

char line1[22] = {0};
char line2[22] = {0};

unsigned char	CountDown = CountDown_PARAM;		//	n half Second Intervals
portTickType 	xDelay;
portBASE_TYPE	TaskCnt;



///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
//	Tasks, variables and Priority definitions
//
void vApplicationTickHook( void );									//	get accel and gyro task
//
//
void Make_It_Balance(void *pvParameters);
#define Balance_TASK_PRIORITY	( tskIDLE_PRIORITY + 2 )			//	Balance task
//
void lcd_out(void *pvParameters);
#define lcd_out_TASK_PRIORITY	( tskIDLE_PRIORITY + 1 )			//	LCD output task

#define max_PRIORITY	( tskIDLE_PRIORITY + 3 )
//
//
//
//
///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
//
//
void WaitForStart(void *pvParameters) 
{
	(void) pvParameters;									//	The parameters are not used.

	portTickType 	xLastWakeTime;
	const 			portTickType xFrequency = 100;			// 5ms periods - .5 seconds
	unsigned char 	WakeSet = FALSE;

	Init_Queues_Semaphores();


	vTaskDelay(2);

	sprintf(line1, "%s", "    Balancing Bot   ");
	sprintf(line2, "%s", " Hit button to start");
	lcd_clear();
	lcd_print(line1);
	lcd_print(line2);

	vTaskDelay(2);
    
//	Cycle for ever
	for( ;; ) {
		if (ButtonPressed()) {								//	Do nothing until start button pressed or has been pressed
//			if (!WakeSet) {
				xLastWakeTime = xTaskGetTickCount();		//	Get base ClockTick for delay timer	
				WakeSet = TRUE;								//	Use WakeSet to only do this the first time through
				sprintf(line1, "%s", "    Countdown       ");
				sprintf(line2,       "        %d          ", CountDown);
				lcd_clear();
				lcd_print(line1);
			}
			if (CountDown > 0) {							//	Loop until the 5 seconds counted down
				sprintf(line2,       "        %d          ", CountDown);
				lcd_clear();
				lcd_print(line1);
				lcd_print(line2);
				if (!(CountDown % 2))						//	We only use countdown so we can blink the LED
					Led(ON);
				else
					Led(OFF);
				vTaskDelayUntil(&xLastWakeTime, xFrequency);
				CountDown--;
			} else {
				//	Seed Random number generator.  (Should be diff each time due to human delay in pressing button)
				srand(xTaskGetTickCount());
				//	We get here because the start button was pressed and we counted down N seconds
				Led(OFF);

/*
				#define MIN_PWM 0
			    #define MAX_PWM 100
			    #define DELAY_MS 500 
			    int count = MIN_PWM;
			    for (;;) {
			        if (count == MAX_PWM)
			            count = MIN_PWM;
			        else
			            count++;
			        sprintf(line1, "Count %4i", count);
			        lcd_clear(); 
			        lcd_print(line1);
			        set_left_motor_pwm(count);
			        set_right_motor_pwm(count);
			        vTaskDelay(DELAY_MS/5);
			    }
*/

				//	Start the rest of required the Tasks running
				xTaskCreate(Make_It_Balance, "Balance", TaskStackSize, NULL, Balance_TASK_PRIORITY, NULL);
				xTaskCreate(lcd_out, "LCD", TaskStackSize, NULL, lcd_out_TASK_PRIORITY, NULL);
				//	Delete Self -- Job well done, not needed anymore
				vTaskDelete(NULL);     
				//vTaskSuspend( NULL );
			}
		//}
	}
}



void Make_It_Balance(void *pvParameters) {
	(void) pvParameters;							//	The parameters are not used.

	int 			cnt;
	portTickType 	xLastWakeTime;
	const 			portTickType xFrequency = 4;	// 	5 milisec ticks - 50 Hz

	cnt = 0;
	ReadIMU = TRUE;									//	Start getting Accel and Gyro
	xLastWakeTime = xTaskGetTickCount();			//	Get base ClockTick for delay timer	

//	Cycle for ever
	for( ;; ) {
	
		if (ReadIMU)
			Read_ADC4_5();							//	read Accel and Gyro

		balance();									//	main balancing act
		vTaskDelayUntil(&xLastWakeTime, xFrequency);//	resolution of one tick period

		cnt++;										//	Heartbeat...
		if (cnt % 6 == 0) {							//	Fast blink the LED when running
			if (LED_ON)								//	Good indication that balance() is alive
				Led(OFF);
			else
				Led(ON);
		}
	}
}




void lcd_out(void *pvParameters) {
	(void) pvParameters;							//	The parameters are not used.

//	Cycle for ever
	for( ;; ) {
		vTaskDelay(50);								// 	5 milisec ticks
													//	refresh LCD at this frequency


		//  The following two lines are used for tuning
		//	uncomment them and comment any others
		//sprintf(line1, "Ac %4.1f Gy %4.1f", (double) accel, (double) (gyro - q_bias));
		//sprintf(line2, "An %4.1f Ra %4.1f", (double) current_angle, (double) rate);

		//	The following line is used for testing the low PWM dead band in the motors
		//	unccoment and comment any others
		sprintf(line1, "L %3d   R %3d", left_motor_torque + left_motor_bias
										,right_motor_torque + right_motor_bias);


		lcd_clear();
		lcd_print(line1);
		lcd_lf();
		lcd_print(line2);
		
	}
}



///////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////
